import serial
import time
from dynamixel_sdk import *

class Ilusha(object):

    PROTOCOL_VERSION = 1.0

    """docstring forIlusha."""

    def __init__(self, portHandler, id):
        super(Ilusha, self).__init__()
        self.portHandler = portHandler
        self.packetHandler = PacketHandler(self.PROTOCOL_VERSION)
        self.id = id

    def write2byte(self, addres, data):
        dxl_comm_result, dxl_error = self.packetHandler.write2ByteTxRx(self.portHandler, self.id , addres, data)
        if dxl_comm_result != COMM_SUCCESS:
            print("%s" % self.packetHandler.getTxRxResult(dxl_comm_result))
        elif dxl_error != 0:
            print("%s" % self.packetHandler.getRxPacketError(dxl_error))

    def write1byte(self, addres, data):
        dxl_comm_result, dxl_error = self.packetHandler.write1ByteTxRx(self.portHandler, self.id , addres, data)
        if dxl_comm_result != COMM_SUCCESS:
            print("%s" % self.packetHandler.getTxRxResult(dxl_comm_result))
        elif dxl_error != 0:
            print("%s" % self.packetHandler.getRxPacketError(dxl_error))

    def read2byte(self, addres):
        a, dxl_comm_result, dxl_error = self.packetHandler.read2ByteTxRx(self.portHandler, self.id, addres)
        if dxl_comm_result != COMM_SUCCESS:
            print("%s" % self.packetHandler.getTxRxResult(dxl_comm_result))
        elif dxl_error != 0:
            print("%s" % self.packetHandler.getRxPacketError(dxl_error))
        return a

    def read1byte(self, addres):
        a, dxl_comm_result, dxl_error = self.packetHandler.read1ByteTxRx(self.portHandler, self.id, addres)
        if dxl_comm_result != COMM_SUCCESS:
            print("%s" % self.packetHandler.getTxRxResult(dxl_comm_result))
        elif dxl_error != 0:
            print("%s" % self.packetHandler.getRxPacketError(dxl_error))
        return a


    def set_enable(self, enable):
        self.write1byte(24, enable)

    def set_PID_P(self, p):
        self.write1byte(28, p)

    def set_PID_I(self, i):
        self.write1byte(27, i)

    def set_PID_D(self, d):
        self.write1byte(26, d)

    def set_max_pwm(self, pwm):
        self.write2byte(14, pwm)

    def set_max_pwm_inc(self, pwm):
        self.write1byte(16, pwm)

    def set_angle(self, angle):
        self.write2byte(30, angle)
        
    def set_angle_offset(self, angle):
        self.write2byte(20, angle)
                  
        
    def set_by_register(self, value, reg, nbytes):
        if nbytes == 1:
            self.write1byte(reg, value)
        elif nbytes == 2:
            self.write2byte(reg, value)
    
    def get_by_register(self, reg, nbytes):
        if nbytes == 1:
            return self.read1byte(reg)
        elif nbytes == 2:
            return self.read2byte(reg)
    
    
    def get_model_number(self):
        return self.read2byte(0)
        
    def get_fw_version(self):
        return self.read1byte(2)
        
    def get_fw_version(self):
        return self.read1byte(2)        

    def get_baundrate(self):
        return self.read1byte(4)
        
    def get_tmax(self):
        return self.read1byte(11)   
        
    def get_vmin(self):
        return self.read1byte(12) 
        
    def get_imax(self):
        return self.read1byte(13) 
        
    def get_max_pwm(self):
        return self.read2byte(14)  

    def get_max_pwm_inc(self):
        return self.read1byte(16)  
        
    def get_imax_k(self):
        return self.read1byte(19)  
        
    def get_offset(self):
        return self.read2byte(20)  

    def get_pidv_d(self):
        return self.read1byte(22) 
        
    def get_pidv_p(self):
        return self.read1byte(23) 
        
    def get_enable(self):
        return self.read2byte(24)
        
    def get_pid_d(self):
        return self.read1byte(26) 
        
    def get_pid_i(self):
        return self.read1byte(27) 
        
    def get_pid_p(self):
        return self.read1byte(28)
        
    def get_pid_i_limit(self):
        return self.read2byte(34)

    def get_temperature(self):
        return self.read1byte(43)      

    def get_angle(self):
        return self.read2byte(36)
        
    def get_goal_angle(self):
        return self.read2byte(30)

    def get_current(self):
        return self.read2byte(40)

    def get_volts(self):
        return self.read1byte(42)
        
    def get_id(self):
        d = []
        for i in range(12):
            d.append(self.read1byte(200+i))
        return d
        
    def save2eeprom(self):
        self.write1byte(47, 1)
        self.write1byte(47, 0)
        
    def set_new_id(self, id):
        self.write1byte(3, id)
        
    def set_speed(self, speed):
        self.write2byte(32, speed)
        
    def ping(self):
        dxl_model_number, dxl_comm_result, dxl_error = self.packetHandler.ping(self.portHandler, self.id)
        if dxl_comm_result != COMM_SUCCESS:
            #print("%s" % self.packetHandler.getTxRxResult(dxl_comm_result))
            return False
        elif dxl_error != 0:
            #print("%s" % self.packetHandler.getRxPacketError(dxl_error))
            return False
        else:
            #print("[ID:%03d] ping Succeeded. Dynamixel model number : %d" % (self.id, dxl_model_number))
            l = self.get_id()
            #print("[ID:%03d] " % (self.id) + ''.join('{:02x}'.format(x) for x in l))
            return True
