import serial
import time
from dynamixel_sdk import *
from ilusha import Ilusha
from rich.console import Console
from rich.table import Table, Column
from rich.progress import track
from rich.live import Live


all_registers = [('0','2','model_number','Номер модели','RE'),
                 ('2','1','fw_version','Версия прошивки','RE'),
                 ('3','1','id','Номер на шине','RWE'),
                 ('4','1','baundrate','Скороcть последовательного интерфейса','RWE'),
                 ('11','1','tmax','Максимальная рабочая температура','RWE'),
                 ('12','1','vmin','Минимальное рабочее напряжение','RWE'),
                 ('13','1','imax','Ограничение тока','RWE'),
                 ('14','2','pwm_max','Максимально возможный шим','RWE'),
                 ('16','1','pwm_max_inc','Ограничение скорости нарастания управляющего сигнала','RWE'),
                 ('19','1','imax_k','Коэффициент ослабления мощности при превышении тока','RWE'),
                 ('20','2','offset','Сдвиг 0 датчика угла ','RWE'),
                 ('22','1','PIDv_d','Дифференциальный коэффициент ПИД регулятора (для управления скоростью)','RWE'),
                 ('23','1','PIDv_p','Пропорциональный коэффициент ПИД регулятора (для управления скоростью)','RWE'),
                 ('24','1','enabled','Включение мотора','RW'),
                 ('26','1','PID_D','Дифференциальный коэффициент ПИД регулятора','RWE'),  
                 ('27','1','PID_I','Интегральный коэффициент ПИД регулятора','RWE'),
                 ('28','1','PID_P','Пропорциональный коэффициент ПИД регулятора','RWE'),
                 ('30','2','goal_angle','Заданный угол поворота','RW'),
                 ('32','2','goal_speed','Заданная скорость (для управления скоростью)','RW'),                 
                 ('34','2','PID_I_limit','Ограничение интегральной части ПИД','RWE'),
                 ('36','2','angle','Текущее значение угла поворота','R'),
                 ('38','2','speed','Текущее значение скорости','R'),
                 ('40','2','I','Текущее значение тока','R'),
                 ('42','1','V','Текущее значение напряжения','R'),
                 ('43','1','T','Текущая температура','R'),                 
                 ('47','1','EEPROM','Команда записи в EEPROM','W')]

def show_reg_map(m, all_registers):
    reg_table = Table(Column(header="Адрес", justify="center", max_width=5),  
                      Column(header="Размер, байт", justify="center", max_width=7), 
                      Column(header="Имя", justify="center", max_width=12), 
                      "Описание", 
                      Column(header="Чтение/ Запись/ EEPROM", justify="center", max_width=7), 
                      Column(header="Значение", justify="center", max_width=8), 
                      show_header=True, title="Карта регистров", show_lines=False,
                      title_style="bold cyan", header_style="bold green")    
    console = Console()               
    for register in all_registers:
        (address, size, name, description, rwe) = register
        reg_table.add_row(address, size, name, description, rwe, str(m.get_by_register(int(address), int(size))))  
    console.print(reg_table)
    


BAUDRATE                    = 115200

portHandler = PortHandler('COM4')
portHandler.setPacketTimeoutMillis(200)

if portHandler.openPort():
    print("Succeeded to open the port")
else:
    print("Failed to open the port")

# Set port baudrate
if portHandler.setBaudRate(BAUDRATE):
    print("Succeeded to change the baudrate")
else:
    print("Failed to change the baudrate")

m = Ilusha(portHandler, 1)


while True:

    id_list=[]
    for i in track(range(1,100), description="[bold cyan]Поиск доступных моторов...", style="black"):
        m.id = i
        if m.ping():
            id_list.append(i)
            
    if id_list:
        print("Доступные id: ", id_list)
        print("Выберете id")
        id_chosen = int(input())
        
        if id_chosen in id_list:
            m.id = id_chosen
            
            show_reg_map(m, all_registers)
            
            menu_info = ("Для изменения регистра введите адрес регистра и новое значение через пробел\n" +
                  "Для сохранения в EEPROM нажмите S\n" +
                  "Для обновления таблицы нажмите U\n" +
                  "Для выхода нажмите Q\n" +
                  "Для выбора мотора с другим id нажмите B\n")
            print(menu_info)    
            
            while True:
                inp = input()                
                command = inp.split(" ")
                
                if len(command) > 1:
                    tup =[reg for reg in all_registers if reg[0] == str(command[0])]
                    
                    if not tup:
                        print("данный регистр недоступен")
                    elif "W" not in tup[0][4]:
                        print("данный регистр недоступен для записи")
                    else:
                        m.set_by_register(int(command[1]), int(command[0]), int(tup[0][1]))
                        print(tup[0][3],"->",command[1],"\n")
                
                elif command[0] == "B":
                    print("\n")
                    break

                elif command[0] == "U":
                    print("\n")
                    show_reg_map(m, all_registers)
                    print(menu_info)                      
                    
                elif command[0] == "Q":
                    quit()
                    
                elif command[0] == "S":
                    print("Данные сохранены в EEPROM\n")
                    m.save2eeprom()
          
        else:
            print("Нет моторов с выбранным id. Пожалуйста, выберете из доступных")
        
    else:
        print("Нет подключенных моторов")
        break





